; This file defines and runs solve-expressions-from-file,
; a procedure that reads expressions from a file and solves 
; them for the given variable.  It then writes the results
; out as a text file, and as input to the dot program which
; produces a plot of the expression tree.

; It is assumed that the procedure (solver x exp) is defined
; in another file named solver.scm.

; The last statement in this file runs the entire process
; with the input coming from "x.txt"
; and the output going to "x-solved.txt" and "x-solved.dot".

(load "solver.scm")
(load "plot-expression.scm")

; read, evaluate, print loop

(define (repl in out plot)
  (let ((x (read in))
        (exp (read in)))
    (if (not (eof-object? exp))
        (begin 
          (display "Solve  " out)
          (write exp out)
          (display " for " out)
          (display x out)
          (newline out)
          (display "Result " out)
          (display (solver x exp) out)
          (newline out)
          (newline out)
          
          (plot-expression exp plot)
          (plot-expression (solver x exp) plot)
          (repl in out plot)
          ))))

(define (solve-expressions-from-file file-root)
  (let ((input-name (string-append file-root ".txt"))
        (output-name (string-append file-root "-solved.txt"))
        (plot-name (string-append file-root "-solved.dot")))
    
    (if (file-exists? output-name)
        (delete-file output-name))
    (if (file-exists? plot-name)
        (delete-file plot-name))
    (let 
        ((port-input (open-input-file input-name))
         (port-output (open-output-file output-name))
         (port-plot (open-output-file plot-name)))
      (repl port-input port-output port-plot)
      (close-input-port port-input)
      (close-output-port port-output)
      (close-output-port port-plot))))

(solve-expressions-from-file "x")
